/*
** Patch module to enable external USB hub on Odys tablets.
** Based on this hacking tutorial:
** http://www.slatedroid.com/topic/20966-fview-7-pro-tablet-rooting-and-fw-discussion/page__view__findpost__p__272987
*/
#include <linux/module.h>       /* for all modules */
#include <linux/init.h>         /* for entry/exit macros */
#include <linux/kernel.h>       /* for printk priority macros */
#include <linux/version.h>      /* for version macros */

#if !(LINUX_VERSION_CODE == KERNEL_VERSION(2,6,32))
#error This module is only for Kernel 2.6.32 and needs to be adapted for other versions!
#endif

#define PATCH_MSG "hubenable: applying %s patch to enable external hub.\n"

static unsigned char *byte;

static int __init hubenable_init(void)
{

  printk(KERN_INFO "Loading module hubenable.\n");

  byte=(unsigned char*)(void *)0xc067769c; /* Check Loox address kernel #90 */
  if( *(byte+0)==0x06 && *(byte+1)==0x00 && *(byte+2)==0x00 && *(byte+3)==0x0a ) /* check for BEQ */
  {
    printk(KERN_INFO PATCH_MSG, "Odys-Loox #90");
    *(byte+3)=0x1a;       /* change to BNE */
    return 0;
  }
  byte=(unsigned char*)(void *)0xc0677770; /* Check Loox address kernel #95 */
  if( *(byte+0)==0x06 && *(byte+1)==0x00 && *(byte+2)==0x00 && *(byte+3)==0x0a ) /* check for BEQ */
  {
    printk(KERN_INFO PATCH_MSG, "Odys-Loox #95");
    *(byte+3)=0x1a;       /* change to BNE */
    return 0;
  }
  byte=(unsigned char*)(void *)0xc066B040; /* Check Xpress address kernel #611 */
  if( *(byte+0)==0x06 && *(byte+1)==0x00 && *(byte+2)==0x00 && *(byte+3)==0x0a ) /* check for BEQ */
  {
    printk(KERN_INFO PATCH_MSG, "Odys-Xpress #611");
    *(byte+3)=0x1a;       /* change to BNE */
    return 0;
  }
  byte=(unsigned char*)(void *)0xc06c0a68; /* Check Cosmo address kernel #147 */
  if( *(byte+0)==0x06 && *(byte+1)==0x00 && *(byte+2)==0x00 && *(byte+3)==0x0a ) /* check for BEQ */
  {
    printk(KERN_INFO PATCH_MSG, "Odys-Cosmo #147");
    *(byte+3)=0x1a;       /* change to BNE */
    return 0;
  }
  byte=(unsigned char*)(void *)0xc06c0a88; /* Check Cosmo address kernel #163 */
  if( *(byte+0)==0x06 && *(byte+1)==0x00 && *(byte+2)==0x00 && *(byte+3)==0x0a ) /* check for BEQ */
  {
    printk(KERN_INFO PATCH_MSG, "Odys-Cosmo #163");
    *(byte+3)=0x1a;       /* change to BNE */
    return 0;
  }
  printk(KERN_ALERT "hubenable: address check for patch failed.\n");
  return -ENXIO;          /* return error 'No such device or address' */
}

static void __exit hubenable_exit(void)
{
  printk(KERN_INFO "Unloading module hubenable.\n");
  if( *(byte+0)==0x06 && *(byte+1)==0x00 && *(byte+2)==0x00 && *(byte+3)==0x1a ) /* check for BNE */
  {
    printk(KERN_INFO "hubenable: reverting patch to enable external hub at: 0x%08x.\n", (unsigned)byte);
    *(byte+3)=0x0a;       /* change to BEQ */
    return;
  }
  return;
}

module_init(hubenable_init);
module_exit(hubenable_exit);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Aiah, fluxflux, wusel");
MODULE_DESCRIPTION("patch module to enable external hubs on Odys Tablets Loox/Xpress/Cosmo");
MODULE_VERSION("0.4");
